use Test2::V1 -ipP;
use Test2::IPC;
skip_all "SQLite driver not available" unless eval { require IPC::Manager::Client::SQLite; IPC::Manager::Client::SQLite->viable };
use lib 't/lib';
use IPC::Manager::Test;
IPC::Manager::Test->run_one(protocol => 'SQLite', test => 'test_service_signal_handling');

done_testing;
